﻿define([
    "dojo/_base/lang",
    "dojo/_base/declare",
    "dojo/Deferred",
    "dojo/dom-style",
    "epi/shell/DialogService",
    "epi/shell/command/builder/ButtonBuilder",
    "epi/shell/widget/dialog/Confirmation",
    "epi-cms/content-approval/ApprovalDefinitionStatus",
    "epi-cms/content-approval/viewmodels/ContentApprovalViewModel",
    // Parent class and mixins
    "dijit/layout/_LayoutWidget",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "epi/shell/widget/_ModelBindingMixin",
    // Resources
    "dojo/text!./templates/ApprovalComponent.html",
    "epi/i18n!epi/nls/episerver.cms.contentapproval.component",
    "epi/i18n!epi/nls/episerver.shared.action",
    // Widgets in template
    "dijit/Toolbar",
    "dijit/form/CheckBox",
    "dijit/form/RadioButton",
    "dijit/form/Select",
    "dijit/form/Form",
    "epi-cms/widget/Breadcrumb",
    "epi-cms/widget/NotificationStatusBar",
    "epi-cms/content-approval/ApprovalStepList"
], function (
    lang,
    declare,
    Deferred,
    domStyle,
    dialogService,
    ButtonBuilder,
    Confirmation,
    ApprovalDefinitionStatus,
    ContentApprovalViewModel,
    // Parent class and mixins
    _LayoutWidget,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    _ModelBindingMixin,
    // Resources
    template,
    localization,
    sharedLocalization
) {

    return declare([_LayoutWidget, _TemplatedMixin, _WidgetsInTemplateMixin, _ModelBindingMixin], {
        // summary:
        //      The view for the content approval component; responsible for creating the content
        //      approval view model and creating child view components.
        // tags:
        //      internal

        // templateString: [protected] String
        //      A string that represents the widget template.
        templateString: template,

        // languageWarningTemplate: [protected] String
        //      HTML template for showing a warning on languageFilter items that has validation issues.
        languageWarningTemplate: " <span class='dijitInline dijitReset dijitIcon epi-iconWarning' title='" + localization.languagefilter.warningmessage + "'></span>",

        // resources: [protected] Object
        //      An object containing resource strings in the current language.
        resources: localization,

        // ApprovalDefinitionStatus: [readonly] epi-cms/content-approval/ApprovalDefinitionStatus
        //      Exposes the different approval definition statuses to the template
        ApprovalDefinitionStatus: ApprovalDefinitionStatus,

        modelBindingMap: {
            currentContentLink: ["breadcrumbContentLink", "notificationContext"],
            isCommentRequiredEnabled: ["isCommentRequiredEnabled"],
            isCommentRequired: ["isCommentRequired"],
            isReadOnly: ["isReadOnly"],
            filterOptions: ["filterOptions"],
            status: ["status"],
            approvalInheritedFrom: ["approvalInheritedFrom"]
        },

        postMixInProperties: function () {
            // summary:
            //      Processing after the parameters to the widget have been read-in, but before the
            //      widget template is instantiated.
            // tags:
            //      protected

            this.inherited(arguments);

            // Create the view model if one has not been injected in the constructor.
            this.model = this.model || new ContentApprovalViewModel();
            this.own(this.model);
        },

        buildRendering: function () {
            // summary:
            //      Build the approval component rendering from the template and then create buttons
            //      for the commands.
            // tags:
            //      protected

            this.inherited(arguments);

            // Generate buttons based on commands available in the view model
            var builder = new ButtonBuilder(),
                commands = this.model.get("commands");

            commands.forEach(function (command) {
                builder.create(command, this.commandNode);
            }, this);
        },

        postCreate: function () {
            // summary:
            //        Processing after the DOM fragment is created
            // tags:
            //        protected

            this.inherited(arguments);

            this.stepList.set("model", this.model);

            this.own(
                this.enableButton.on("click", this._updateStatus.bind(this, ApprovalDefinitionStatus.enabled)),
                this.disableButton.on("click", this._updateStatus.bind(this, ApprovalDefinitionStatus.disabled)),
                this.inheritButton.on("click", this._updateStatus.bind(this, ApprovalDefinitionStatus.inherited)),
                this.demandCommentButton.on("click", this._updateDemandComment.bind(this)),
                this.languageFilter.on("change", this._updateSelectedLanguage.bind(this))
            );
        },

        updateView: function () {
            // summary:
            //		This will be invoked by WidgetSwitcher when displaying Approval component.
            // tags:
            //		public

            this._focus();
        },

        _updateSelectedLanguage: function (language) {
            this.model.set("selectedLanguage", language);
        },

        _setNotificationContextAttr: function (notificationContext) {
            // summary:
            //      Set the validation message query to filter what messages on
            //      messageService should display in the NotificationStatusBar.
            // tags:
            //      private

            this._set("notificationContext", notificationContext);

            this.validation.set("notificationContext", {
                contextTypeName: "epi.cms.approval",
                contextId: this.model.currentContentLink
            });
        },

        _setBreadcrumbContentLinkAttr: function (breadcrumbContentLink) {
            // summary:
            //      Sets the contentLink for the breadcrumb
            // tags:
            //      private

            this._set("breadcrumbContentLink", breadcrumbContentLink);
            this.breadcrumb.set("contentLink", this.model.currentContentLink);
        },

        _setStatusAttr: function (status) {
            this._set("status", status);
            this.form.set("value", {status: status});

            this._focus();

            // Toggle the visibility of the step list, the filter node and the disabled warning node
            domStyle.set(this.stepList.domNode, "display", this.model.get("isEnabled") ? "" : "none");
            domStyle.set(this.filterNode, "display", this.model.get("isEnabled") && !!this.model.get("filterOptions") ? "" : "none");
            this._updateWarningMessage();
        },

        _focus: function () {
            this.enableButton.get("checked") && this.enableButton.focus();
            this.disableButton.get("checked") && this.disableButton.focus();
            this.inheritButton.get("checked") && this.inheritButton.focus();
        },

        _setApprovalInheritedFromAttr: function (approvalInheritedFrom) {
            this._set("approvalInheritedFrom", approvalInheritedFrom);
            this._updateWarningMessage();
        },

        _updateWarningMessage: function () {
            domStyle.set(this.warningNode, "display", !this.model.get("isEnabled") || this.model.get("isInherited") ? "" : "none");
            domStyle.set(this.goToInheritedSequence, "display", this.model.get("isInherited") ? "" : "none");
            this.warningMessage.textContent = this.model.getWarningMessage();
        },

        _setIsCommentRequiredAttr: function (isCommentRequired) {
            this._set("isCommentRequired", isCommentRequired);
            this.demandCommentButton.set("checked", isCommentRequired);
        },

        _setIsCommentRequiredEnabledAttr: function (isCommentRequiredEnabled) {
            this._set("isCommentRequiredEnabled", isCommentRequiredEnabled);
            this.demandCommentButton.set("disabled", !isCommentRequiredEnabled);
        },

        _setIsReadOnlyAttr: function (isReadOnly) {
            this._set("isReadOnly", isReadOnly);
            this.enableButton.set("disabled", isReadOnly);
            this.disableButton.set("disabled", isReadOnly);
            this.inheritButton.set("disabled", isReadOnly);
        },

        _setFilterOptionsAttr: function (filterOptions) {
            // summary:
            //      Shows or hides the filter widget and also sets its options.
            // tags:
            //      private

            this._set("filterOptions", filterOptions);

            var showFilter = !!filterOptions && this.model.get("isEnabled");
            domStyle.set(this.filterNode, "display", showFilter ? "" : "none");
            if (!filterOptions) {
                return;
            }

            // Clone the object so changes to it won't affect the model.
            filterOptions = lang.clone(filterOptions);
            // Keep track of the selected options. If it's "All Languages" use an empty string.
            var selectedLanguage = this.model.get("selectedLanguage") || "";
            filterOptions.forEach(function (filter) {
                if (!filter.isValid && this.model.get("showValidations")) {
                    filter.label += this.languageWarningTemplate;
                }

                filter.selected = selectedLanguage === filter.value;
            }, this);

            // Reset the filter options, and the selected item to update the displayed value.
            this.languageFilter.set("options", filterOptions);
            this.languageFilter.set("value", selectedLanguage);
        },

        _goToInheritedSequence: function () {
            this.model.goToInheritedSequence();
        },

        _updateStatus: function (nextStatus) {
            var promise;
            switch (nextStatus) {
                case ApprovalDefinitionStatus.inherited:
                    promise = this._confirmInherit();
                    break;
                case ApprovalDefinitionStatus.enabled:
                    this.model.enable();
                    promise = new Deferred().resolve();
                    break;
                case ApprovalDefinitionStatus.disabled:
                    promise = this._confirmDisable();
                    break;
            }

            // Reset the status if the user cancels the dialog.
            promise.otherwise(this.set.bind(this, "status", this.status));
        },

        _updateDemandComment: function (e) {
            var checked = e.target.checked;
            this.model.set("isCommentRequired", checked);
            this.demandCommentButton.set("checked", checked);
        },

        _confirmDisable: function () {
            var settings = {
                title: this.resources.disable.title,
                description: this.resources.disable.description,
                confirmActionText: sharedLocalization.disable,
                cancelActionText: sharedLocalization.cancel
            };

            return dialogService.confirmation(settings)
                .then(this.model.disable.bind(this.model));
        },

        _confirmInherit: function () {
            var settings = {
                title: this.resources.inherit.title,
                description: this.resources.inherit.description,
                confirmActionText: this.resources.inherit.okbutton,
                cancelActionText: sharedLocalization.cancel,
                setFocusOnConfirmButton: false
            };

            return dialogService.confirmation(settings)
                .then(this.model.inheritDefinition.bind(this.model));
        }
    });
});
